// extract frames for the voxel growth movie from the given images and corner coordinates

// frame dimensions and offset from baseline
// (dependent on the resolution of the micrographs)
int Width = 320;
int Height = 240;
int xOffset = -95;
int yOffset = 10;

// these are the names of the micrograph image files, and their asociated corner coordinate files
String[] filenames = 
{
  "IMG_2704",
  "IMG_2705",
  "IMG_2706",
  "IMG_2707",
  "IMG_2708",
  "IMG_2709",
  "IMG_2710",
  "IMG_2711",
  "IMG_2712",
  "IMG_2713",
  "IMG_2714",
  "IMG_2715",
  "IMG_2716",
  "IMG_2717",
  "IMG_2718",
};

void setup() {

  int frameNum = 0;
  for (int i = 0; i < filenames.length; i++) 
  {  
    // load the original image
    PImage img;
    img = loadImage("input/" + filenames[i] + ".JPG");
    
    // load its associated corner coordinates
    Table coords = loadTable("input/" + filenames[i] + ".txt", "header, tsv");
    
    for(int j = 0; j < coords.getRowCount() - 1; j++)
    {
      TableRow left = coords.getRow(j);
      TableRow right = coords.getRow(j + 1);
      
      // find y position of the frame
      float y = 10.0 - (left.getFloat("y") + right.getFloat("y")) / 2.0;
      y *= img.height / 10.0;
      y -= yOffset;
      
      // find  x position of the frame
      float x = (left.getFloat("x") + right.getFloat("x")) / 2.0;
      x *= img.width / 10.0;
      x += xOffset - (Width / 2.0);

      // copy the frame into a new image, flipping it vertically
      PImage frame = createImage(Width, Height, RGB);
      for(int Y = 0; Y < Height; Y++)
      {
        int srcOff = (Y + (int)(y + 0.5)) * img.width + (int)(x + 0.5);
        int dstOff = (Height - 1 - Y) * Width;
        for(int X = 0; X < Width; X++)
          frame.pixels[dstOff + X] = img.pixels[srcOff + X];
      }
     
      // and save it
      frame.save("output/" + (frameNum < 10 ? "0" : "") + frameNum + ".png");
      println(frameNum + " " + filenames[i]);
      frameNum++;
    }
  }
  println("done");
}